#include "xyza2pipe.h"

char header[PIPE_HEADERSIZE], _header_[PIPE_HEADERSIZE];
char axislabel[4][MAXASSNAME + 1];
char axisname[4][MAXASSNAME + 1];

short dimension, _dimension, byteswap, headersize;
short swapdata = 0, swappar = 0, usrlabel = 0, usrshift = 0;
short leftcar = 0, extleft = 0, adjcar = 0, adjh2o = 0;

short blocksize[4];
short usrphase[4] = { 0, 0, 0, 0 };

int datasize[4], _datasize[4];
int unitsize[4];

float obsfreq[4];
float spcenter[4];
float origfreq[4];
float spwidth[4];
float usrcenter[4];
float phase[4][2];

float **matrix2d;
float ***matrix3d;
float ****matrix4d;

const float PIPE_HEADER[3] = { PIPE_HEADER_0, PIPE_HEADER_1, PIPE_HEADER_2 };

const char *usage[] = {
 "addvnmr2pipe -i/--in1 inTemplate1(2D/3D/4D) -j/--in2 inTemplate2(2D/3D/4D) > stdout\n", "Optional Arguments:\n",
     " -p/--pdir1   A directory that includes VNMR procpar file1. (default=\".\")\n",
     " -q/--pdir2   A directory that includes VNMR procpar file2. (default=\".\")\n",
     " -a/--add inTemplate1 + inTemplate2 (default)\n", " -s/--sub inTemplate1 - inTemplate2\n",
     " -m/--mul inTemplate1 * inTemplate2\n", " -d/--div inTemplate1 / inTemplate2\n",
     " --c1   Scale factor for inTemplate1 (default=1.0)\n", " --c2   Scale factor for inTemplate2 (default=1.0)\n",
     " --xLAB X-Axis Label\n", " --yLAB Y-Axis Label\n", " --zLAB Z-Axis Label\n", " --xCAR X-Axis Center [ppm]\n",
     " --yCAR Y-Axis Center [ppm]\n", " --zCAR Z-Axis Center [ppm]\n", ""
};

int main(int argc, char *argv[])
{
 char axis_option, filename1[MAXLONGNAME], filename2[MAXLONGNAME], pardir1[MAXLONGNAME], pardir2[MAXLONGNAME];
 char *suffix1 = { "1" }, *suffix2 = {
 "2"};
 int l, opt_idx = 0, nfiles1 = 1, nfiles2 = 1;
 float c1 = 1.0, c2 = 1.0;
 enum enum_combine_opr opr_code = COMBINE_ADD;

 static struct option options[] = {
  {"in1", required_argument, 0, 'i'},
  {"in2", required_argument, 0, 'j'},
  {"pdir1", required_argument, 0, 'p'},
  {"pdir2", required_argument, 0, 'q'},
  {"c1", required_argument, 0, 3},
  {"c2", required_argument, 0, 4},
  {"add", no_argument, 0, 'a'},
  {"sub", no_argument, 0, 's'},
  {"mul", no_argument, 0, 'm'},
  {"div", no_argument, 0, 'd'},
  {"xLAB", required_argument, 0, 6},
  {"yLAB", required_argument, 0, 7},
  {"zLAB", required_argument, 0, 8},
  {"xCAR", required_argument, 0, 10},
  {"yCAR", required_argument, 0, 11},
  {"zCAR", required_argument, 0, 12},
  {0, 0, 0, 0}
 };

 filename1[0] = filename2[0] = 0;
 strcpy(pardir1, ".");
 strcpy(pardir2, ".");
 axis_option = 'x';

 for (l = 0; l < 4; l++)
  usrcenter[l] = NULLPPM;

 while (1) {
  l = getopt_long(argc, argv, "i:j:p:q:asmd", options, &opt_idx);

  if (l == -1)
   break;

  switch (l) {
  case 0:
   if (options[opt_idx].flag != 0)
    break;
   fprintf(stderr, "option %s", options[opt_idx].name);
   if (optarg)
    fprintf(stderr, " with arg %s", optarg);
   fputc('\n', stderr);
   break;
  case 'i':
   strncpy(filename1, optarg, MAXLONGNAME);
   break;
  case 'j':
   strncpy(filename2, optarg, MAXLONGNAME);
   break;
  case 'p':
   strncpy(pardir1, optarg, MAXLONGNAME);
   break;
  case 'q':
   strncpy(pardir2, optarg, MAXLONGNAME);
   break;
  case 'a':
   opr_code = COMBINE_ADD;
   break;
  case 's':
   opr_code = COMBINE_SUB;
   break;
  case 'm':
   opr_code = COMBINE_MUL;
   break;
  case 'd':
   opr_code = COMBINE_DIV;
   break;
  case 3:
   c1 = atof(optarg);
   break;
  case 4:
   c2 = atof(optarg);
   break;
  case 6:
   usrlabel = 1;
   strncpy(axislabel[0], optarg, MAXASSNAME);
   break;
  case 7:
   usrlabel = 1;
   strncpy(axislabel[1], optarg, MAXASSNAME);
   break;
  case 8:
   usrlabel = 1;
   strncpy(axislabel[2], optarg, MAXASSNAME);
   break;
  case 10:
   usrshift = 1;
   usrcenter[0] = atof(optarg);
   break;
  case 11:
   usrshift = 1;
   usrcenter[1] = atof(optarg);
   break;
  case 12:
   usrshift = 1;
   usrcenter[2] = atof(optarg);
   break;
  case '?':			/* getopt_long already printed an error message. */
   break;
  }
 }

 if (argc == 3) {
  strncpy(filename1, argv[1], MAXLONGNAME);
  strncpy(filename2, argv[2], MAXLONGNAME);
 }

 else if (optind < argc) {
  fprintf(stderr, "non-option ARGV-elements: ");
  while (optind < argc)
   fprintf(stderr, "%s ", argv[optind++]);
  fputc('\n', stderr);
 }

 if (filename1[0] == 0 || filename2[0] == 0) {
  l = 0;
  while (strcmp(usage[l], "") != 0)
   fprintf(stderr, "%s", usage[l++]);

  return 1;
 }

 if (checkvnmr(filename1, pardir1, &nfiles1, suffix1) != 0)
  return 1;

 _dimension = dimension;

 for (l = 0; l < dimension; l++)
  _datasize[l] = datasize[l];

 if (checkvnmr(filename2, pardir2, &nfiles2, suffix2) != 0)
  return 1;

 if (dimension != _dimension) {
  fprintf(stderr, "addvnmr2pipe error: unmatch dimension number.\n");
  return 1;
 }

 for (l = 0; l < dimension; l++) {
  if (datasize[l] != _datasize[l]) {
   fprintf(stderr, "addvnmr2pipe error: unmatch data size.\n");
   return 1;
  }
 }

 cnvhdr(axis_option, 'f');

 if (isatty(STDOUT_FILENO)) {
  fprintf(stderr, "addvnmr2pipe error: output to terminal.\n");
  return 1;
 }

 switch (dimension) {
 case 2:
  return pushaddvnmr2d(filename1, filename2, pardir1, pardir2, c1, c2, opr_code);
  break;

 case 3:
  return pushaddvnmr3d(filename1, filename2, pardir1, pardir2, nfiles1, nfiles2, suffix1, suffix2, c1, c2, opr_code);
  break;
 }

 return 0;
}
