/* libMath: Mathematical library

 Bugs can be reported to yokochi@pharm.hokudai.ac.jp.

 This work is a part of Olivia development project.
   http://fermi.pharm.hokudai.ac.jp/olivia/
*/

#ifndef __LIB_MATH__
#define __LIB_MATH__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <math.h>

#ifndef __OLIVIA_IRIX__

#include <stdint.h>

#endif

typedef struct {
 float r, i;
} complex;

typedef struct {
 double r, i;
} doublecomplex;

typedef struct {
 complex x, y;
} hypercomplex;

typedef struct {
 doublecomplex x, y;
} doublehypercomplex;

typedef struct {
 float r, i, j, k;
} quaternion;

double powint(double x, int n);

float gasdev(int32_t idum);

float ran1(int32_t idum);
float ran4(int32_t idum);

float gammln(float xx);
float factln(int n);
float factrl(int n);

float perm(int n, int k);
float bico(int n, int k);

float erfcc(float x);

float betai(float a, float b, float x);
float betacf(float a, float b, float x);

/* linear correlation coefficient (Pearson's r) */

void pearsn(float x[], float y[], const size_t n, float *r, float *z, float *prob);

/* rank correlation coefficient (Spearman's rs) */

void spear(float x[], float y[], const size_t n, float *rs, float *zd, float *probrs, float *d, float *probd);

/* rank correlation coefficient (Kendall's tau) */

void kendl1(float x[], float y[], const size_t n, float *tau, float *z, float *prob);

/* Geni's coefficient */

void gini(float x[], const size_t n, float *gc);

/* cubic spline interpolation */

void spline(float *x, float *y, const size_t n, float *y2);
float splint(float *xa, float *ya, float *y2a, const size_t n, float x);

/* search function minimum (brent's method) */

float brent(float ax, float bx, float cx, float (*f) (float), float tol, float *xmin);
float brent_thrdsafe(float ax, float bx, float cx, float (*f) (int, float), float tol, float *xmin, int thrd_id);

/* complex */

complex Cadd(complex a, complex b);
complex Csub(complex a, complex b);
complex Cmul(complex a, complex b);
complex Conj(complex z);
complex Cdiv(complex a, complex b);
float Cabs(complex z);
complex Csqrt(complex z);
complex RCmul(float x, complex a);
complex Cexp(complex z);

/* doublecomplex */

doublecomplex DCadd(doublecomplex a, doublecomplex b);
doublecomplex DCsub(doublecomplex a, doublecomplex b);
doublecomplex DCmul(doublecomplex a, doublecomplex b);
doublecomplex DConj(doublecomplex z);
doublecomplex DCdiv(doublecomplex a, doublecomplex b);
double DCabs(doublecomplex z);
doublecomplex DCsqrt(doublecomplex z);
doublecomplex RDCmul(double x, doublecomplex a);
doublecomplex DCexp(doublecomplex z);

/* hypercomplex */

hypercomplex HCadd(hypercomplex a, hypercomplex b);
hypercomplex HCsub(hypercomplex a, hypercomplex b);
hypercomplex HCmul(hypercomplex a, hypercomplex b);
hypercomplex HConj(hypercomplex z);
hypercomplex HCtp(hypercomplex z);

/* quaternion */

quaternion Qadd(quaternion a, quaternion b);
quaternion Qsub(quaternion a, quaternion b);
quaternion Qmul(quaternion a, quaternion b);
quaternion Qonj(quaternion z);

#endif
