#include "libMatrix.h"

char *charmalloc1d(const size_t size1)
{
 char *p;

 p = (char *) (malloc(size1 * sizeof(char)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in charmalloc1d().\n");
  exit(EXIT_FAILURE);
 }

 return p;
}

char **charmalloc2d(const size_t size1, const size_t size2)
{
 int i;
 char **p;

 p = (char **) (malloc(size1 * sizeof(char *)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in charmalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (char *) (malloc(size1 * size2 * sizeof(char)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in charmalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 for (i = 1; i < size1; i++)
  p[i] = p[i - 1] + size2;

 return p;
}

char ***charmalloc3d(const size_t size1, const size_t size2, const size_t size3)
{
 int i, j;
 char ***p;

 p = (char ***) (malloc(size1 * sizeof(char **)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in charmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (char **) (malloc(size1 * size2 * sizeof(char *)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in charmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (char *) (malloc(size1 * size2 * size3 * sizeof(char)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in charmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 for (j = 1; j < size2; j++)
  p[0][j] = p[0][j - 1] + size3;

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  for (j = 1; j < size2; j++)
   p[i][j] = p[i][j - 1] + size3;
 }

 return p;
}

char ****charmalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4)
{
 int i, j, k;
 char ****p;

 p = (char ****) (malloc(size1 * sizeof(char ***)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in charmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (char ***) (malloc(size1 * size2 * sizeof(char **)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in charmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (char **) (malloc(size1 * size2 * size3 * sizeof(char *)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in charmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0][0] = (char *) (malloc(size1 * size2 * size3 * size4 * sizeof(char)));

 if (p[0][0][0] == NULL) {
  fprintf(stderr, "Short of memory in charmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 for (k = 1; k < size3; k++)
  p[0][0][k] = p[0][0][k - 1] + size4;

 for (j = 1; j < size2; j++) {
  p[0][j] = p[0][j - 1] + size3;
  p[0][j][0] = p[0][j - 1][0] + size3 * size4;
  for (k = 1; k < size3; k++)
   p[0][j][k] = p[0][j][k - 1] + size4;
 }

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  p[i][0][0] = p[i - 1][0][0] + size2 * size3 * size4;

  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i][0][k - 1] + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i][j - 1] + size3;
   p[i][j][0] = p[i][j - 1][0] + size3 * size4;
  }
 }

 for (i = 1; i < size1; i++) {
  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i - 1][0][k - 1] + size2 * size3 * size4 + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i - 1][j - 1] + size2 * size3 + size3;
   p[i][j][0] = p[i - 1][j - 1][0] + size2 * size3 * size4 + size3 * size4;
   for (k = 1; k < size3; k++)
    p[i][j][k] = p[i - 1][j][k - 1] + size2 * size3 * size4 + size4;
  }
 }

 return p;
}

void free_charmatrix1d(char *p)
{
 if (p == NULL)
  return;

 free((char *) (p));

 p = NULL;
}

void free_charmatrix2d(char **p)
{
 if (p == NULL)
  return;

 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_charmatrix3d(char ***p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_charmatrix4d(char ****p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0][0]));
 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

short *smalloc1d(const size_t size1)
{
 short *p;

 p = (short *) (malloc(size1 * sizeof(short)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in smalloc1d().\n");
  exit(EXIT_FAILURE);
 }

 return p;
}

short **smalloc2d(const size_t size1, const size_t size2)
{
 int i;
 short **p;

 p = (short **) (malloc(size1 * sizeof(short *)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in smalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (short *) (malloc(size1 * size2 * sizeof(short)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in smalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 for (i = 1; i < size1; i++)
  p[i] = p[i - 1] + size2;

 return p;
}

short ***smalloc3d(const size_t size1, const size_t size2, const size_t size3)
{
 int i, j;
 short ***p;

 p = (short ***) (malloc(size1 * sizeof(short **)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in smalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (short **) (malloc(size1 * size2 * sizeof(short *)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in smalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (short *) (malloc(size1 * size2 * size3 * sizeof(short)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in smalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 for (j = 1; j < size2; j++)
  p[0][j] = p[0][j - 1] + size3;

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  for (j = 1; j < size2; j++)
   p[i][j] = p[i][j - 1] + size3;
 }

 return p;
}

short ****smalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4)
{
 int i, j, k;
 short ****p;

 p = (short ****) (malloc(size1 * sizeof(short ***)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in smalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (short ***) (malloc(size1 * size2 * sizeof(short **)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in smalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (short **) (malloc(size1 * size2 * size3 * sizeof(short *)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in smalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0][0] = (short *) (malloc(size1 * size2 * size3 * size4 * sizeof(short)));

 if (p[0][0][0] == NULL) {
  fprintf(stderr, "Short of memory in smalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 for (k = 1; k < size3; k++)
  p[0][0][k] = p[0][0][k - 1] + size4;

 for (j = 1; j < size2; j++) {
  p[0][j] = p[0][j - 1] + size3;
  p[0][j][0] = p[0][j - 1][0] + size3 * size4;
  for (k = 1; k < size3; k++)
   p[0][j][k] = p[0][j][k - 1] + size4;
 }

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  p[i][0][0] = p[i - 1][0][0] + size2 * size3 * size4;

  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i][0][k - 1] + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i][j - 1] + size3;
   p[i][j][0] = p[i][j - 1][0] + size3 * size4;
  }
 }

 for (i = 1; i < size1; i++) {
  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i - 1][0][k - 1] + size2 * size3 * size4 + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i - 1][j - 1] + size2 * size3 + size3;
   p[i][j][0] = p[i - 1][j - 1][0] + size2 * size3 * size4 + size3 * size4;
   for (k = 1; k < size3; k++)
    p[i][j][k] = p[i - 1][j][k - 1] + size2 * size3 * size4 + size4;
  }
 }

 return p;
}

void free_smatrix1d(short *p)
{
 if (p == NULL)
  return;

 free((char *) (p));

 p = NULL;
}

void free_smatrix2d(short **p)
{
 if (p == NULL)
  return;

 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_smatrix3d(short ***p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_smatrix4d(short ****p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0][0]));
 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

int *imalloc1d(const size_t size1)
{
 int *p;

 p = (int *) (malloc(size1 * sizeof(int)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in imalloc1d().\n");
  exit(EXIT_FAILURE);
 }

 return p;
}

int **imalloc2d(const size_t size1, const size_t size2)
{
 int i;
 int **p;

 p = (int **) (malloc(size1 * sizeof(int *)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in imalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (int *) (malloc(size1 * size2 * sizeof(int)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in imalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 for (i = 1; i < size1; i++)
  p[i] = p[i - 1] + size2;

 return p;
}

int ***imalloc3d(const size_t size1, const size_t size2, const size_t size3)
{
 int i, j;
 int ***p;

 p = (int ***) (malloc(size1 * sizeof(int **)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in imalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (int **) (malloc(size1 * size2 * sizeof(int *)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in imalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (int *) (malloc(size1 * size2 * size3 * sizeof(int)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in imalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 for (j = 1; j < size2; j++)
  p[0][j] = p[0][j - 1] + size3;

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  for (j = 1; j < size2; j++)
   p[i][j] = p[i][j - 1] + size3;
 }

 return p;
}

int ****imalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4)
{
 int i, j, k;
 int ****p;

 p = (int ****) (malloc(size1 * sizeof(int ***)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in imalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (int ***) (malloc(size1 * size2 * sizeof(int **)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in imalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (int **) (malloc(size1 * size2 * size3 * sizeof(int *)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in imalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0][0] = (int *) (malloc(size1 * size2 * size3 * size4 * sizeof(int)));

 if (p[0][0][0] == NULL) {
  fprintf(stderr, "Short of memory in imalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 for (k = 1; k < size3; k++)
  p[0][0][k] = p[0][0][k - 1] + size4;

 for (j = 1; j < size2; j++) {
  p[0][j] = p[0][j - 1] + size3;
  p[0][j][0] = p[0][j - 1][0] + size3 * size4;
  for (k = 1; k < size3; k++)
   p[0][j][k] = p[0][j][k - 1] + size4;
 }

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  p[i][0][0] = p[i - 1][0][0] + size2 * size3 * size4;

  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i][0][k - 1] + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i][j - 1] + size3;
   p[i][j][0] = p[i][j - 1][0] + size3 * size4;
  }
 }

 for (i = 1; i < size1; i++) {
  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i - 1][0][k - 1] + size2 * size3 * size4 + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i - 1][j - 1] + size2 * size3 + size3;
   p[i][j][0] = p[i - 1][j - 1][0] + size2 * size3 * size4 + size3 * size4;
   for (k = 1; k < size3; k++)
    p[i][j][k] = p[i - 1][j][k - 1] + size2 * size3 * size4 + size4;
  }
 }

 return p;
}

void free_imatrix1d(int *p)
{
 if (p == NULL)
  return;

 free((char *) (p));

 p = NULL;
}

void free_imatrix2d(int **p)
{
 if (p == NULL)
  return;

 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_imatrix3d(int ***p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_imatrix4d(int ****p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0][0]));
 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

float *fmalloc1d(const size_t size1)
{
 float *p;

 p = (float *) (malloc(size1 * sizeof(float)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in fmalloc1d().\n");
  exit(EXIT_FAILURE);
 }

 return p;
}

float **fmalloc2d(const size_t size1, const size_t size2)
{
 int i;
 float **p;

 p = (float **) (malloc(size1 * sizeof(float *)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in fmalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (float *) (malloc(size1 * size2 * sizeof(float)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in fmalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 for (i = 1; i < size1; i++)
  p[i] = p[i - 1] + size2;

 return p;
}

float ***fmalloc3d(const size_t size1, const size_t size2, const size_t size3)
{
 int i, j;
 float ***p;

 p = (float ***) (malloc(size1 * sizeof(float **)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in fmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (float **) (malloc(size1 * size2 * sizeof(float *)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in fmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (float *) (malloc(size1 * size2 * size3 * sizeof(float)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in fmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 for (j = 1; j < size2; j++)
  p[0][j] = p[0][j - 1] + size3;

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  for (j = 1; j < size2; j++)
   p[i][j] = p[i][j - 1] + size3;
 }

 return p;
}

float ****fmalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4)
{
 int i, j, k;
 float ****p;

 p = (float ****) (malloc(size1 * sizeof(float ***)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in fmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (float ***) (malloc(size1 * size2 * sizeof(float **)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in fmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (float **) (malloc(size1 * size2 * size3 * sizeof(float *)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in fmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0][0] = (float *) (malloc(size1 * size2 * size3 * size4 * sizeof(float)));

 if (p[0][0][0] == NULL) {
  fprintf(stderr, "Short of memory in fmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 for (k = 1; k < size3; k++)
  p[0][0][k] = p[0][0][k - 1] + size4;

 for (j = 1; j < size2; j++) {
  p[0][j] = p[0][j - 1] + size3;
  p[0][j][0] = p[0][j - 1][0] + size3 * size4;
  for (k = 1; k < size3; k++)
   p[0][j][k] = p[0][j][k - 1] + size4;
 }

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  p[i][0][0] = p[i - 1][0][0] + size2 * size3 * size4;

  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i][0][k - 1] + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i][j - 1] + size3;
   p[i][j][0] = p[i][j - 1][0] + size3 * size4;
  }
 }

 for (i = 1; i < size1; i++) {
  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i - 1][0][k - 1] + size2 * size3 * size4 + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i - 1][j - 1] + size2 * size3 + size3;
   p[i][j][0] = p[i - 1][j - 1][0] + size2 * size3 * size4 + size3 * size4;
   for (k = 1; k < size3; k++)
    p[i][j][k] = p[i - 1][j][k - 1] + size2 * size3 * size4 + size4;
  }
 }

 return p;
}

void free_fmatrix1d(float *p)
{
 if (p == NULL)
  return;

 free((char *) (p));

 p = NULL;
}

void free_fmatrix2d(float **p)
{
 if (p == NULL)
  return;

 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_fmatrix3d(float ***p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_fmatrix4d(float ****p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0][0]));
 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

double *dmalloc1d(const size_t size1)
{
 double *p;

 p = (double *) (malloc(size1 * sizeof(double)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in dmalloc1d().\n");
  exit(EXIT_FAILURE);
 }

 return p;
}

double **dmalloc2d(const size_t size1, const size_t size2)
{
 int i;
 double **p;

 p = (double **) (malloc(size1 * sizeof(double *)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in dmalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (double *) (malloc(size1 * size2 * sizeof(double)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in dmalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 for (i = 1; i < size1; i++)
  p[i] = p[i - 1] + size2;

 return p;
}

double ***dmalloc3d(const size_t size1, const size_t size2, const size_t size3)
{
 int i, j;
 double ***p;

 p = (double ***) (malloc(size1 * sizeof(double **)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in dmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (double **) (malloc(size1 * size2 * sizeof(double *)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in dmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (double *) (malloc(size1 * size2 * size3 * sizeof(double)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in dmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 for (j = 1; j < size2; j++)
  p[0][j] = p[0][j - 1] + size3;

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  for (j = 1; j < size2; j++)
   p[i][j] = p[i][j - 1] + size3;
 }

 return p;
}

double ****dmalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4)
{
 int i, j, k;
 double ****p;

 p = (double ****) (malloc(size1 * sizeof(double ***)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in dmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (double ***) (malloc(size1 * size2 * sizeof(double **)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in dmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (double **) (malloc(size1 * size2 * size3 * sizeof(double *)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in dmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0][0] = (double *) (malloc(size1 * size2 * size3 * size4 * sizeof(double)));

 if (p[0][0][0] == NULL) {
  fprintf(stderr, "Short of memory in dmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 for (k = 1; k < size3; k++)
  p[0][0][k] = p[0][0][k - 1] + size4;

 for (j = 1; j < size2; j++) {
  p[0][j] = p[0][j - 1] + size3;
  p[0][j][0] = p[0][j - 1][0] + size3 * size4;
  for (k = 1; k < size3; k++)
   p[0][j][k] = p[0][j][k - 1] + size4;
 }

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  p[i][0][0] = p[i - 1][0][0] + size2 * size3 * size4;

  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i][0][k - 1] + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i][j - 1] + size3;
   p[i][j][0] = p[i][j - 1][0] + size3 * size4;
  }
 }

 for (i = 1; i < size1; i++) {
  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i - 1][0][k - 1] + size2 * size3 * size4 + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i - 1][j - 1] + size2 * size3 + size3;
   p[i][j][0] = p[i - 1][j - 1][0] + size2 * size3 * size4 + size3 * size4;
   for (k = 1; k < size3; k++)
    p[i][j][k] = p[i - 1][j][k - 1] + size2 * size3 * size4 + size4;
  }
 }

 return p;
}

void free_dmatrix1d(double *p)
{
 if (p == NULL)
  return;

 free((char *) (p));

 p = NULL;
}

void free_dmatrix2d(double **p)
{
 if (p == NULL)
  return;

 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_dmatrix3d(double ***p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_dmatrix4d(double ****p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0][0]));
 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

complex *cmalloc1d(const size_t size1)
{
 complex *p;

 p = (complex *) (malloc(size1 * sizeof(complex)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in cmalloc1d().\n");
  exit(EXIT_FAILURE);
 }

 return p;
}

complex **cmalloc2d(const size_t size1, const size_t size2)
{
 int i;
 complex **p;

 p = (complex **) (malloc(size1 * sizeof(complex *)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in cmalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (complex *) (malloc(size1 * size2 * sizeof(complex)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in cmalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 for (i = 1; i < size1; i++)
  p[i] = p[i - 1] + size2;

 return p;
}

complex ***cmalloc3d(const size_t size1, const size_t size2, const size_t size3)
{
 int i, j;
 complex ***p;

 p = (complex ***) (malloc(size1 * sizeof(complex **)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in cmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (complex **) (malloc(size1 * size2 * sizeof(complex *)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in cmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (complex *) (malloc(size1 * size2 * size3 * sizeof(complex)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in cmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 for (j = 1; j < size2; j++)
  p[0][j] = p[0][j - 1] + size3;

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  for (j = 1; j < size2; j++)
   p[i][j] = p[i][j - 1] + size3;
 }

 return p;
}

complex ****cmalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4)
{
 int i, j, k;
 complex ****p;

 p = (complex ****) (malloc(size1 * sizeof(complex ***)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in cmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (complex ***) (malloc(size1 * size2 * sizeof(complex **)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in cmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (complex **) (malloc(size1 * size2 * size3 * sizeof(complex *)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in cmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0][0] = (complex *) (malloc(size1 * size2 * size3 * size4 * sizeof(complex)));

 if (p[0][0][0] == NULL) {
  fprintf(stderr, "Short of memory in cmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 for (k = 1; k < size3; k++)
  p[0][0][k] = p[0][0][k - 1] + size4;

 for (j = 1; j < size2; j++) {
  p[0][j] = p[0][j - 1] + size3;
  p[0][j][0] = p[0][j - 1][0] + size3 * size4;
  for (k = 1; k < size3; k++)
   p[0][j][k] = p[0][j][k - 1] + size4;
 }

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  p[i][0][0] = p[i - 1][0][0] + size2 * size3 * size4;

  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i][0][k - 1] + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i][j - 1] + size3;
   p[i][j][0] = p[i][j - 1][0] + size3 * size4;
  }
 }

 for (i = 1; i < size1; i++) {
  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i - 1][0][k - 1] + size2 * size3 * size4 + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i - 1][j - 1] + size2 * size3 + size3;
   p[i][j][0] = p[i - 1][j - 1][0] + size2 * size3 * size4 + size3 * size4;
   for (k = 1; k < size3; k++)
    p[i][j][k] = p[i - 1][j][k - 1] + size2 * size3 * size4 + size4;
  }
 }

 return p;
}

void free_cmatrix1d(complex * p)
{
 if (p == NULL)
  return;

 free((char *) (p));

 p = NULL;
}

void free_cmatrix2d(complex ** p)
{
 if (p == NULL)
  return;

 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_cmatrix3d(complex *** p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_cmatrix4d(complex **** p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0][0]));
 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

doublecomplex *dcmalloc1d(const size_t size1)
{
 doublecomplex *p;

 p = (doublecomplex *) (malloc(size1 * sizeof(doublecomplex)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in dcmalloc1d().\n");
  exit(EXIT_FAILURE);
 }

 return p;
}

doublecomplex **dcmalloc2d(const size_t size1, const size_t size2)
{
 int i;
 doublecomplex **p;

 p = (doublecomplex **) (malloc(size1 * sizeof(doublecomplex *)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in dcmalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (doublecomplex *) (malloc(size1 * size2 * sizeof(doublecomplex)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in dcmalloc2d().\n");
  exit(EXIT_FAILURE);
 }

 for (i = 1; i < size1; i++)
  p[i] = p[i - 1] + size2;

 return p;
}

doublecomplex ***dcmalloc3d(const size_t size1, const size_t size2, const size_t size3)
{
 int i, j;
 doublecomplex ***p;

 p = (doublecomplex ***) (malloc(size1 * sizeof(doublecomplex **)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in dcmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (doublecomplex **) (malloc(size1 * size2 * sizeof(doublecomplex *)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in dcmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (doublecomplex *) (malloc(size1 * size2 * size3 * sizeof(doublecomplex)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in dcmalloc3d().\n");
  exit(EXIT_FAILURE);
 }

 for (j = 1; j < size2; j++)
  p[0][j] = p[0][j - 1] + size3;

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  for (j = 1; j < size2; j++)
   p[i][j] = p[i][j - 1] + size3;
 }

 return p;
}

doublecomplex ****dcmalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4)
{
 int i, j, k;
 doublecomplex ****p;

 p = (doublecomplex ****) (malloc(size1 * sizeof(doublecomplex ***)));

 if (p == NULL) {
  fprintf(stderr, "Short of memory in dcmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0] = (doublecomplex ***) (malloc(size1 * size2 * sizeof(doublecomplex **)));

 if (p[0] == NULL) {
  fprintf(stderr, "Short of memory in dcmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0] = (doublecomplex **) (malloc(size1 * size2 * size3 * sizeof(doublecomplex *)));

 if (p[0][0] == NULL) {
  fprintf(stderr, "Short of memory in dcmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 p[0][0][0] = (doublecomplex *) (malloc(size1 * size2 * size3 * size4 * sizeof(doublecomplex)));

 if (p[0][0][0] == NULL) {
  fprintf(stderr, "Short of memory in dcmalloc4d().\n");
  exit(EXIT_FAILURE);
 }

 for (k = 1; k < size3; k++)
  p[0][0][k] = p[0][0][k - 1] + size4;

 for (j = 1; j < size2; j++) {
  p[0][j] = p[0][j - 1] + size3;
  p[0][j][0] = p[0][j - 1][0] + size3 * size4;
  for (k = 1; k < size3; k++)
   p[0][j][k] = p[0][j][k - 1] + size4;
 }

 for (i = 1; i < size1; i++) {
  p[i] = p[i - 1] + size2;
  p[i][0] = p[i - 1][0] + size2 * size3;
  p[i][0][0] = p[i - 1][0][0] + size2 * size3 * size4;

  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i][0][k - 1] + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i][j - 1] + size3;
   p[i][j][0] = p[i][j - 1][0] + size3 * size4;
  }
 }

 for (i = 1; i < size1; i++) {
  for (k = 1; k < size3; k++)
   p[i][0][k] = p[i - 1][0][k - 1] + size2 * size3 * size4 + size4;

  for (j = 1; j < size2; j++) {
   p[i][j] = p[i - 1][j - 1] + size2 * size3 + size3;
   p[i][j][0] = p[i - 1][j - 1][0] + size2 * size3 * size4 + size3 * size4;
   for (k = 1; k < size3; k++)
    p[i][j][k] = p[i - 1][j][k - 1] + size2 * size3 * size4 + size4;
  }
 }

 return p;
}

void free_dcmatrix1d(doublecomplex * p)
{
 if (p == NULL)
  return;

 free((char *) (p));

 p = NULL;
}

void free_dcmatrix2d(doublecomplex ** p)
{
 if (p == NULL)
  return;

 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_dcmatrix3d(doublecomplex *** p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void free_dcmatrix4d(doublecomplex **** p)
{
 if (p == NULL)
  return;

 free((char *) (p[0][0][0]));
 free((char *) (p[0][0]));
 free((char *) (p[0]));
 free((char *) (p));

 p = NULL;
}

void tred2(double **a, const size_t n, double d[], double e[])
{
 int l, k, j, i;
 double scale, hh, h, g, f;

 for (i = n; i > 1; i--) {
  l = i - 1;
  h = scale = 0.0;

  if (l > 1) {

   for (k = 1; k <= l; k++)
    scale += fabs(a[i][k]);

   if (scale == 0.0)
    e[i] = a[i][l];

   else {

    for (k = 1; k <= l; k++) {
     a[i][k] /= scale;

     h += a[i][k] * a[i][k];
    }

    f = a[i][l];
    g = (f >= 0.0 ? -sqrt(h) : sqrt(h));

    e[i] = scale * g;
    h -= f * g;

    a[i][l] = f - g;

    f = 0.0;
    for (j = 1; j <= l; j++) {
     a[j][i] = a[i][j] / h;

     g = 0.0;
     for (k = 1; k <= j; k++)
      g += a[j][k] * a[i][k];

     for (k = j + 1; k <= l; k++)
      g += a[k][j] * a[i][k];

     e[j] = g / h;

     f += e[j] * a[i][j];
    }

    hh = f / (h + h);

    for (j = 1; j <= l; j++) {
     f = a[i][j];

     e[j] = g = e[j] - hh * f;

     for (k = 1; k <= j; k++)
      a[j][k] -= f * e[k] + g * a[i][k];
    }
   }
  }

  else
   e[i] = a[i][l];

  d[i] = h;
 }

 d[1] = e[1] = 0.0;

 for (i = 1; i <= n; i++) {
  l = i - 1;

  if (d[i] != 0.0) {

   for (j = 1; j <= l; j++) {
    g = 0.0;

    for (k = 1; k <= l; k++)
     g += a[i][k] * a[k][j];

    for (k = 1; k <= l; k++)
     a[k][j] -= g * a[k][i];
   }
  }

  d[i] = a[i][i];
  a[i][i] = 1.0;

  for (j = 1; j <= l; j++)
   a[j][i] = a[i][j] = 0.0;
 }
}

#define	MAXROTATION	100
#define	SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))

void tqli(double d[], double e[], const size_t n, double **z)
{
 int m, l, iter, i, k;
 double s, r, p, g, f, dd, c, b;

 for (i = 2; i <= n; i++)
  e[i - 1] = e[i];

 e[n] = 0.0;

 for (l = 1; l <= n; l++) {
  iter = 0;

  do {
   for (m = l; m <= n - 1; m++) {
    dd = fabs(d[m]) + fabs(d[m + 1]);

    if ((double) (fabs(e[m]) + dd) == dd)
     break;
   }

   if (m != l) {
    if (iter++ == MAXROTATION) {
     fprintf(stderr, "Too many iterations in tqli().\n");
     exit(EXIT_FAILURE);
    }

    g = (d[l + 1] - d[l]) / (2.0 * (e[l]));
    r = hypot(g, 1.0);
    g = d[m] - d[l] + e[l] / (g + SIGN(r, g));

    s = c = 1.0;
    p = 0.0;

    for (i = m - 1; i >= l; i--) {
     f = s * e[i];
     b = c * e[i];

     e[i + 1] = (r = hypot(f, g));

     if (r == 0.0) {
      d[i + 1] -= p;
      e[m] = 0.0;
      break;
     }

     s = f / r;
     c = g / r;

     g = d[i + 1] - p;
     r = (d[i] - g) * s + 2.0 * c * b;

     d[i + 1] = g + (p = s * r);

     g = c * r - b;

     for (k = 1; k <= n; k++) {
      f = z[k][i + 1];

      z[k][i + 1] = s * z[k][i] + c * f;
      z[k][i] = c * z[k][i] - s * f;
     }
    }

    if (r == 0.0 && i >= l)
     continue;

    d[l] -= p;

    e[l] = g;
    e[m] = 0.0;
   }
  }

  while (m != l);
 }
}

#define RADIX	2.0

void balanc(double **a, const size_t n)
{
 int last = 0, j, i;
 double s, r, g, f, c, sqrdx = RADIX * RADIX;

 while (last == 0) {
  last = 1;

  for (i = 1; i <= n; i++) {
   r = c = 0.0;

   for (j = 1; j <= n; j++) {
    if (j != i) {
     c += fabs(a[j][i]);
     r += fabs(a[i][j]);
    }
   }

   if (c != 0.0 && r != 0.0) {
    g = r / RADIX;
    f = 1.0;
    s = c + r;

    while (c < g) {
     f *= RADIX;
     c *= sqrdx;
    }

    g = r * RADIX;

    while (c > g) {
     f /= RADIX;
     c /= sqrdx;
    }

    if ((c + r) / f < 0.95 * s) {
     last = 0;

     g = 1.0 / f;

     for (j = 1; j <= n; j++)
      a[i][j] *= g;

     for (j = 1; j <= n; j++)
      a[j][i] *= g;
    }
   }
  }
 }
}

#define SWAP(g,h) {y=(g);(g)=(h);(h)=y;}

void elmhes(double **a, const size_t n)
{
 int m, j, i;
 double y, x;

 for (m = 2; m < n; m++) {
  x = 0.0;
  i = m;

  for (j = m; j <= n; j++) {
   if (fabs(a[j][m - 1]) > fabs(x)) {
    x = a[j][m - 1];
    i = j;
   }
  }

  if (i != m) {
   for (j = m - 1; j <= n; j++)
    SWAP(a[i][j], a[m][j]);

   for (j = 1; j <= n; j++)
    SWAP(a[j][i], a[j][m]);
  }

  if (x != 0.0) {

   for (i = m + 1; i <= n; i++) {

    if ((y = a[i][m - 1]) != 0.0) {
     y /= x;

     a[i][m - 1] = y;

     for (j = m; j <= n; j++)
      a[i][j] -= y * a[m][j];

     for (j = 1; j <= n; j++)
      a[j][m] += y * a[j][i];
    }
   }
  }
 }
}

void sgecpy(const size_t n, float *A, float *B)
{
 memcpy((void *) A, (void *) B, n * n * sizeof(float));
}

#define EPS	8.640266e+08
#define eps	1.0/8.640266e+08

int sgecmp(const size_t n, float *A, float *B)
{
 int i, nn = n * n;

 for (i = 0; i < nn; i++) {
  if (fabs(A[i] - B[i]) > eps)
   return 1;
 }

 return 0;
}

void sgeexp(const size_t n, float *A, float *B)
{
 int i, j, m, p, q, n1 = n + 1, nn = n * n;
 float fac = 1.0;
 float *C, *D;

 C = (float *) (calloc(nn, sizeof(float)));
 D = (float *) (malloc(nn * sizeof(char)));

 for (i = q = 0; i < n; i++, q += n1)
  C[q] = 1.0;

 memcpy((void *) B, (void *) C, nn * sizeof(float));

 for (m = 1;; m += 2) {

  fac *= (float) (m);

  if (fac > EPS)
   break;

  /* D = C * A (m = odd) */

  _sgemm_(n, C, A, D);

  /* B += A^m / m! */

  for (i = q = 0; i < n; i++, q += n) {
   for (j = 0, p = q; j < n; j++, p++)
    B[p] += D[p] / fac;
  }

  fac *= (float) (m + 1);

  if (fac > EPS)
   break;

  /* C = D * A (m = even) */

  _sgemm_(n, D, A, C);

  /* B += A^m / m! */

  for (i = q = 0; i < n; i++, q += n) {
   for (j = 0, p = q; j < n; j++, p++)
    B[p] += C[p] / fac;
  }
 }

 free((char *) (C));
 free((char *) (D));
}

void _sgemm_(const size_t n, float *A, float *B, float *C)
{
 int i, j, k, l, m;
 float tot;

 for (i = l = 0; i < n; i++, l += n) {
  for (j = 0; j < n; j++) {

   tot = 0.0;
   for (k = m = 0; k < n; k++, m += n)
    tot += A[l + k] * B[j + m];

   C[l + j] = tot;
  }
 }
}
