/* libMatrix: Memory allocation/free library

 Bugs can be reported to yokochi@pharm.hokudai.ac.jp.

 This work is a part of Olivia development project.
   http://fermi.pharm.hokudai.ac.jp/olivia/
*/

#ifndef __LIB_MATRIX__
#define __LIB_MATRIX__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#include "libMath.h"

char *charmalloc1d(const size_t size1);
char **charmalloc2d(const size_t size1, const size_t size2);
char ***charmalloc3d(const size_t size1, const size_t size2, const size_t size3);
char ****charmalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4);

void free_charmatrix1d(char *p);
void free_charmatrix2d(char **p);
void free_charmatrix3d(char ***p);
void free_charmatrix4d(char ****p);

short *smalloc1d(const size_t size1);
short **smalloc2d(const size_t size1, const size_t size2);
short ***smalloc3d(const size_t size1, const size_t size2, const size_t size3);
short ****smalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4);

void free_smatrix1d(short *p);
void free_smatrix2d(short **p);
void free_smatrix3d(short ***p);
void free_smatrix4d(short ****p);

int *imalloc1d(const size_t size1);
int **imalloc2d(const size_t size1, const size_t size2);
int ***imalloc3d(const size_t size1, const size_t size2, const size_t size3);
int ****imalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4);

void free_imatrix1d(int *p);
void free_imatrix2d(int **p);
void free_imatrix3d(int ***p);
void free_imatrix4d(int ****p);

float *fmalloc1d(const size_t size1);
float **fmalloc2d(const size_t size1, const size_t size2);
float ***fmalloc3d(const size_t size1, const size_t size2, const size_t size3);
float ****fmalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4);

void free_fmatrix1d(float *p);
void free_fmatrix2d(float **p);
void free_fmatrix3d(float ***p);
void free_fmatrix4d(float ****p);

double *dmalloc1d(const size_t size1);
double **dmalloc2d(const size_t size1, const size_t size2);
double ***dmalloc3d(const size_t size1, const size_t size2, const size_t size3);
double ****dmalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4);

void free_dmatrix1d(double *p);
void free_dmatrix2d(double **p);
void free_dmatrix3d(double ***p);
void free_dmatrix4d(double ****p);

complex *cmalloc1d(const size_t size1);
complex **cmalloc2d(const size_t size1, const size_t size2);
complex ***cmalloc3d(const size_t size1, const size_t size2, const size_t size3);
complex ****cmalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4);

void free_cmatrix1d(complex * p);
void free_cmatrix2d(complex ** p);
void free_cmatrix3d(complex *** p);
void free_cmatrix4d(complex **** p);

doublecomplex *dcmalloc1d(const size_t size1);
doublecomplex **dcmalloc2d(const size_t size1, const size_t size2);
doublecomplex ***dcmalloc3d(const size_t size1, const size_t size2, const size_t size3);
doublecomplex ****dcmalloc4d(const size_t size1, const size_t size2, const size_t size3, const size_t size4);

void free_dcmatrix1d(doublecomplex * p);
void free_dcmatrix2d(doublecomplex ** p);
void free_dcmatrix3d(doublecomplex *** p);
void free_dcmatrix4d(doublecomplex **** p);

void tred2(double **a, const size_t n, double d[], double e[]);
void tqli(double d[], double e[], const size_t n, double **z);

void balanc(double **a, const size_t n);
void elmhes(double **a, const size_t n);

void sgecpy(const size_t n, float *A, float *B);
int sgecmp(const size_t n, float *A, float *B);
void sgeexp(const size_t n, float *A, float *B);
void _sgemm_(const size_t n, float *A, float *B, float *C);

#endif
