/* libString: C string processing library

 Bugs can be reported to yokochi@pharm.hokudai.ac.jp.

 This work is a part of Olivia development project.
   http://fermi.pharm.hokudai.ac.jp/olivia/
*/

#ifndef __LIB_STRING__
#define __LIB_STRING__

#ifndef __OLIVIA_IRIX__

#include <stdint.h>

#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>
#include <ctype.h>
#include <time.h>

#define MAXVARS		64
#define MAXCHAR		1024

void path2uscore(char *path);

int column_of(const char *str);
int row_of(const char *str);
int quote_of(char *str);

int is_integer(const char *str);

#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__)

#ifndef HAVE_STRNSTR
#define HAVE_STRNSTR
#endif

#ifndef HAVE_STRCASESTR
#define HAVE_STRCASESTR
#endif

#endif

#if defined(__USE_GNU)

#ifndef HAVE_STRCASESTR
#define HAVE_STRCASESTR
#endif

#endif

/* Non-standard <string.h> routines: derived from FreeBSD */

#ifndef HAVE_STRNSTR

char *strnstr(const char *haystack, const char *needle, size_t n);

#endif

/* Non-standard <string.h> routines: derived from glibc-2.5 */

#ifndef HAVE_STRCASESTR

char *strcasestr(const char *haystack, const char *needle);

#endif

int strsearch(const char *haystack, const char *needle);
int strcasesearch(const char *haystack, const char *needle);

int strmatch(const char *haystack, const char *needle);
int strnmatch(const char *haystack, const char *needle, size_t n);
int strcasematch(const char *haystack, const char *needle);

int strmatchcount(const char *haystack, const char *needle);
int strcasematchcount(const char *haystack, const char *needle);

char *struppercpy(char *dst, const char *src);
char *strlowercpy(char *dst, const char *src);
char *strclasscpy(char *dst, const char *src);
char *strunquotecpy(char *dst, const char *src);
char *strseqoffcpy(char *dst, const char *src);

char *dirnamecpy(char *dir, const char *path);
char *basenamecpy(char *baes, const char *path);

char *strspacefillcpy(char *dst, const char *src, size_t len);
char *strspacefill(char *src, size_t len);

char *strreplacecpy(char *dst, const char *src, const char *key, const char *trl);
char *strreplace(char *src, const char *key, const char *trl);

char *strcenteringcpy(char *dst, const char *src, size_t len);
char *strquarteringcpy(char *dst, const char *src, size_t len);

void charwrite2bin(FILE * fp, int8_t a);
void swrite2bin(FILE * fp, int16_t a);
void iwrite2bin(FILE * fp, int32_t a);
void lwrite2bin(FILE * fp, long a);
void fwrite2bin(FILE * fp, float a);

void spwrite2bin(FILE * fp, int16_t * a, size_t n);
void ipwrite2bin(FILE * fp, int32_t * a, size_t n);
void lpwrite2bin(FILE * fp, long *a, size_t n);
void fpwrite2bin(FILE * fp, float *a, size_t n);

void charwrite2mem(void *mem, int8_t a);
void swrite2mem(void *mem, int16_t a);
void iwrite2mem(void *mem, int32_t a);
void lwrite2mem(void *mem, long a);
void fwrite2mem(void *mem, float a);

void swrite2bin_swap(FILE * fp, int16_t a, int swap);
void iwrite2bin_swap(FILE * fp, int32_t a, int swap);
void lwrite2bin_swap(FILE * fp, long a, int swap);
void fwrite2bin_swap(FILE * fp, float a, int swap);

void spwrite2bin_swap(FILE * fp, int16_t * a, size_t n, int swap);
void ipwrite2bin_swap(FILE * fp, int32_t * a, size_t n, int swap);
void lpwrite2bin_swap(FILE * fp, long *a, size_t n, int swap);
void fpwrite2bin_swap(FILE * fp, float *a, size_t n, int swap);

void swrite2mem_swap(void *mem, int16_t a, int swap);
void iwrite2mem_swap(void *mem, int32_t a, int swap);
void lwrite2mem_swap(void *mem, long a, int swap);
void fwrite2mem_swap(void *mem, float a, int swap);

int is_big_endian();
int is_little_endian();

int is_big_endian_float(char *mem, float real);
int is_little_endian_float(char *mem, float real);

void swapbyte(unsigned int unit_size, unsigned int mem_size, char *mem);

char *parse2str(char *p_char, const char *str, unsigned int start, unsigned int stop);
int *parse2int(int *p_int, const char *str, unsigned int start, unsigned int stop);
float *parse2float(float *p_float, const char *str, unsigned int start, unsigned int stop);

int line2arg(const char *str, unsigned char fs_code, char *argv[]);
int line4arg(const char *str, unsigned char fs_code, char *argv[], unsigned char start_code, unsigned char stop_code);

/* NOTE THAT FOLLOWING FUNCTIONS ARE THREAD-UNSAFE FUNCTIONS */

void alloc_arg(unsigned int thrd_id, int size, char *argv[]);
void free_arg(unsigned int thrd_id);

char *_dirname_(const char *path);
char *_basename_(const char *path);

#endif
