#include "xyza2pipe.h"
#include "vendorpar.h"

enum enum_data_type {
 HYPERCOMPLEX_DATA, COMPLEX_DATA, REAL_DATA
};

#define VHDR_BEGIN	(32/sizeof(int))
#define VHDR_MID	(56/sizeof(int))

int openvnmr2d(char spectra2d[], char pardir[], float **mat2d)
{
 struct stat _stat;
 FILE *fp;
 char string[MAXCHAR];
 int i, j, _i, _j, i_;
 int block_size, block_i, block_j, block_id;
 int offset_i, offset_j, offset, position;
 int vhead_begin, vhead_mid;
 long size = 0;
 complex mix, ret;
 enum enum_data_type data_type = HYPERCOMPLEX_DATA;
 hypercomplex data_hc, data_tp, data_px;
 hypercomplex phi, _phase, _phase_x;
 datafilehead fheader;

 if ((fp = fopen(spectra2d, "r")) == NULL) {
  fprintf(stderr, "Spectra file %s: Couldn't open.\n", spectra2d);
  return 1;
 }

 stat(spectra2d, &_stat);
 size = _stat.st_size;

 fread(&fheader, sizeof(datafilehead), 1, fp);

 if (byteswap != 0) {
  swapbyte(sizeof(int32_t), sizeof(int32_t) * 6, (char *) &fheader);
  swapbyte(sizeof(short), sizeof(short) * 2, (char *) &fheader + sizeof(int32_t) * 6);
  swapbyte(sizeof(int32_t), sizeof(int32_t), (char *) &fheader + sizeof(int32_t) * 6 + sizeof(short) * 2);
 }

/* CHECK FILE SIZE */
 if (size != fheader.nblocks * fheader.bbytes + sizeof(datafilehead)) {
  fprintf(stderr, "Spectra file %s: Partially broken. (Actual=%d Expected=%d)\n", spectra2d, (int) (size),
      (int) (fheader.nblocks * fheader.bbytes + sizeof(datafilehead)));
  goto escape;
 }

 if ((fheader.status & S_COMPLEX) != 0) {
  if ((fheader.status & S_HYPERCOMPLEX) != 0) {
   data_type = HYPERCOMPLEX_DATA;
  } else {
   data_type = COMPLEX_DATA;
   fprintf(stderr, "complex data is not supported.\n");
   goto escape;
  }
 } else {
  data_type = REAL_DATA;
  fprintf(stderr, "real data is not supported.\n");
  goto escape;
 }

 if (usrphase[0] == 0 || data_type != HYPERCOMPLEX_DATA) {
  get_varian_parameter(dimension, pardir, "rp", 0, string);
  phase[0][0] = atof(string);
  get_varian_parameter(dimension, pardir, "lp", 0, string);
  phase[0][1] = atof(string);
 }

 if (usrphase[1] == 0 || data_type != HYPERCOMPLEX_DATA) {
  get_varian_parameter(dimension, pardir, "rp1", 0, string);
  phase[1][0] = atof(string);
  get_varian_parameter(dimension, pardir, "lp1", 0, string);
  phase[1][1] = atof(string);
 }

 fwrite2mem(header + 436, phase[0][0]);
 fwrite2mem(header + 440, phase[0][1]);
 fwrite2mem(header + 980, phase[1][0]);
 fwrite2mem(header + 984, phase[1][1]);

#ifdef __DEBUG__

 fprintf(stderr, "phase settings: rp= %.1f lp= %.1f [deg]\n", phase[0][0], phase[0][1]);
 fprintf(stderr, "phase settings: rp1=%.1f lp1=%.1f [deg]\n\n", phase[1][0], phase[1][1]);

#endif

 vhead_begin = sizeof(datafilehead);
 vhead_mid = fheader.nbheaders * sizeof(datablockhead);

 switch (data_type) {
 case HYPERCOMPLEX_DATA:
  phi.x.r = phi.y.r = 0.0;

  blocksize[0] = fheader.ntraces * 4;
  blocksize[1] = fheader.np;

  block_size = fheader.ntraces * fheader.np;

  for (j = 0; j < datasize[1]; j++) {
   _j = j;
   block_j = (int) (_j / blocksize[1]);
   offset_j = _j - block_j * blocksize[1];

   phi.y.i = -M_PI / 180.0 * (phase[1][0] + (float) (j) / (float) (datasize[1]) * phase[1][1]);

   _phase.y = Cexp(phi.y);

   for (i = 0; i < datasize[0]; i++) {
    _i = i * 4;
    block_i = (int) (_i / blocksize[0]);
    offset_i = _i - block_i * blocksize[0];

    phi.x.i = -M_PI / 180.0 * (phase[0][0] + (float) (i) / (float) (datasize[0]) * phase[0][1]);

    _phase.x = Cexp(phi.x);

    i_ = i;
    if ((i % fheader.ntraces) >= 0 && (i % fheader.ntraces) < 3) {
     i_ = i + fheader.ntraces;

     if (i_ < 0)
      i_ += datasize[0];
     else if (i_ >= datasize[0])
      i_ -= datasize[0];
    }

    block_id = block_i + block_j * unitsize[0];

    offset = offset_i + offset_j * blocksize[0];

    position = offset + block_id * block_size;

    headersize = vhead_begin + vhead_mid * (int) (position / block_size);

    fseek(fp, (long) (headersize + position * sizeof(float)), SEEK_SET);

    fread(&data_tp, sizeof(char), sizeof(hypercomplex), fp);

    if (byteswap != 0)
     swapbyte(sizeof(float), sizeof(hypercomplex), (char *) (&data_tp));

    data_hc = HCtp(data_tp);

    _phase_x.x = _phase.x;
    _phase_x.y = _phase.x;

    data_px = HCmul(data_hc, _phase_x);

    mix.r = data_px.x.r;

    if ((i % fheader.ntraces) == 3) {
     _i = ((i + fheader.ntraces * 3) % datasize[0]) * 4;
     block_i = (int) (_i / blocksize[0]);
     offset_i = _i - block_i * blocksize[0];

     block_id = block_i + block_j * unitsize[0];

     offset = offset_i + offset_j * blocksize[0];

     position = offset + block_id * block_size;

     headersize = vhead_begin + vhead_mid * (int) (position / block_size);

     fseek(fp, (long) (headersize + position * sizeof(float)), SEEK_SET);

     fread(&data_tp, sizeof(char), sizeof(hypercomplex), fp);

     if (byteswap != 0)
      swapbyte(sizeof(float), sizeof(hypercomplex), (char *) (&data_tp));

     data_hc = HCtp(data_tp);

     data_px.y = Cmul(data_hc.y, _phase.x);
    }

    mix.i = -data_px.y.r;

    ret = Cmul(mix, _phase.y);

    mat2d[j][i_] = ret.r;
   }
  }

  break;
 }

 fclose(fp);

 return 0;

escape:fclose(fp);

 return 1;
}

int openvnmr3d(char spectra3d[], char pardir[], const int nfiles, float ***mat3d)
{
 struct stat _stat;
 FILE *fp;
 int i, j, k, _i, _j, _k;
 int block_size, block_i, block_j, block_k, block_id;
 int offset_i, offset_j, offset_k, offset, position;
 int vhead_begin, vhead_mid;
 long size = 0;
 enum enum_data_type data_type = HYPERCOMPLEX_DATA;

#ifdef __DEBUG__

 char string[MAXCHAR];
 float rp[3], lp[3];

#endif

/*
 complex mix, ret;
 hypercomplex data_hc, data_tp, data_px;
 hypercomplex phi, _phase, _phase_x;
*/

 datafilehead fheader;

 if ((fp = fopen(spectra3d, "r")) == NULL) {
  fprintf(stderr, "Spectra file %s: Couldn't open.\n", spectra3d);
  return 1;
 }

 stat(spectra3d, &_stat);
 size = _stat.st_size;

 fread(&fheader, sizeof(datafilehead), 1, fp);

/* CHECK FILE SIZE */
 if (size != fheader.nblocks * fheader.bbytes + sizeof(datafilehead)) {
  fprintf(stderr, "Spectra file %s: Partially broken. (Actual=%d Expected=%d)\n", spectra3d, (int) (size),
      (int) (fheader.nblocks * fheader.bbytes + sizeof(datafilehead)));
  goto escape;
 }

 if ((fheader.status & S_COMPLEX) != 0) {
  if ((fheader.status & S_HYPERCOMPLEX) != 0) {
   data_type = HYPERCOMPLEX_DATA;
   fprintf(stderr, "hypercomplex data is not supported.\n");
   goto escape;
  } else {
   data_type = COMPLEX_DATA;
   fprintf(stderr, "complex data is not supported.\n");
   goto escape;
  }
 } else {
  data_type = REAL_DATA;
 }

#ifdef __DEBUG__

 get_varian_parameter(dimension, pardir, "rp", 0, string);
 phase[0][0] = atof(string);
 get_varian_parameter(dimension, pardir, "lp", 0, string);
 phase[0][1] = atof(string);

 get_varian_parameter(dimension, pardir, "rp1", 0, string);
 phase[1][0] = atof(string);
 get_varian_parameter(dimension, pardir, "lp1", 0, string);
 phase[1][1] = atof(string);

 get_varian_parameter(dimension, pardir, "rp2", 0, string);
 phase[2][0] = atof(string);
 get_varian_parameter(dimension, pardir, "lp2", 0, string);
 phase[2][1] = atof(string);

 fwrite2mem(header + 436, phase[0][0]);
 fwrite2mem(header + 440, phase[0][1]);
 fwrite2mem(header + 980, phase[1][0]);
 fwrite2mem(header + 984, phase[1][1]);
 fwrite2mem(header + 240, phase[2][0]);
 fwrite2mem(header + 244, phase[2][1]);

 fprintf(stderr, "phase settings: rp= %.1f lp= %.1f [deg]\n", phase[0][0], phase[0][1]);
 fprintf(stderr, "phase settings: rp1=%.1f lp1=%.1f [deg]\n", phase[1][0], phase[1][1]);
 fprintf(stderr, "phase settings: rp2=%.1f lp2=%.1f [deg]\n\n", phase[2][0], phase[2][1]);

#endif

 vhead_begin = sizeof(datafilehead);
 vhead_mid = fheader.nbheaders * sizeof(datablockhead);

 switch (data_type) {
 case REAL_DATA:
  blocksize[1] = fheader.ntraces;
  blocksize[0] = fheader.np / nfiles;
  blocksize[2] = nfiles;

  block_size = fheader.ntraces * fheader.np;

  for (k = 0; k < datasize[2]; k++) {
   _k = k;
   block_k = (int) (_k / blocksize[2]);
   offset_k = _k - block_k * blocksize[2];

   for (j = 0; j < datasize[1]; j++) {
    _j = j;
    block_j = (int) (_j / blocksize[1]);
    offset_j = _j - block_j * blocksize[1];

    for (i = 0; i < datasize[0]; i++) {
     _i = i;
     block_i = (int) (_i / blocksize[0]);
     offset_i = _i - block_i * blocksize[0];

     block_id = block_i + (block_j + block_k * unitsize[1]) * unitsize[0];

     offset = offset_i + (offset_j + offset_k * blocksize[1]) * blocksize[0];

     position = offset + block_id * block_size;

     headersize = vhead_begin + vhead_mid * (int) (position / block_size);

     fseek(fp, (long) (headersize + position * sizeof(float)), SEEK_SET);

     fread(&(mat3d[k][j][i]), sizeof(char), sizeof(float), fp);
    }
   }
  }

  break;
 }

 fclose(fp);

 return 0;

escape:fclose(fp);

 return 1;
}
