#include "xyza2pipe.h"

int openxyza2d(char spectra2d[], float **mat2d)
{
 struct stat _stat;
 FILE *fp;
 char buffer[PIPE_HEADERSIZE];
 int i, j;
 long size = 0;
 float *vp;

 if ((fp = fopen(spectra2d, "r")) == NULL) {
  fprintf(stderr, "Spectra file %s: Couldn't open.\n", spectra2d);
  return 1;
 }

 stat(spectra2d, &_stat);
 size = _stat.st_size;

 fread(buffer, sizeof(char), size > PIPE_HEADERSIZE ? PIPE_HEADERSIZE : size, fp);

/* CHECK FILE SIZE */
 vp = (float *) ((void *) (buffer));

 byteswap = 0;

 if (*vp != PIPE_HEADER[0] || *(vp + 2) != PIPE_HEADER[2]) {
  byteswap = 1;
  swapbyte(sizeof(float), size > PIPE_HEADERSIZE ? PIPE_HEADERSIZE : size, buffer);

  vp = (float *) ((void *) (buffer));

  if (*vp != PIPE_HEADER[0] || *(vp + 2) != PIPE_HEADER[2]) {
   fprintf(stderr, "Spectra file: Not NMRPipe file.\n");
   goto escape;
  }
 }

 if (size != sizeof(float) * datasize[0] * datasize[1] + PIPE_HEADERSIZE) {
  fprintf(stderr, "Spectra file %s: Partially broken. (Actual=%d Expected=%d)\n", spectra2d, (int) (size),
      (int) (sizeof(float)) * datasize[0] * datasize[1] + PIPE_HEADERSIZE);
  goto escape;
 }

 for (j = 0; j < datasize[1]; j++) {
  for (i = 0; i < datasize[0]; i++) {
   fseek(fp, (long) (PIPE_HEADERSIZE + (i + j * datasize[0]) * sizeof(float)), SEEK_SET);

   fread(&(mat2d[j][i]), sizeof(char), sizeof(float), fp);
  }
 }

 if (byteswap != 0)
  swapbyte(sizeof(float), datasize[0] * datasize[1] * sizeof(float), (char *) (&(mat2d[0][0])));

 fclose(fp);

 return 0;

escape:fclose(fp);

 return 1;
}

int openxyza3d(char spectra3d[], const int z, float **mat2d)
{
 struct stat _stat;
 FILE *fp;
 char filename[MAXLONGNAME], buffer[PIPE_HEADERSIZE];
 int i, j;
 long size = 0;
 float *vp;

 sprintf(filename, spectra3d, z + 1);

 if ((fp = fopen(filename, "r")) == NULL) {
  fprintf(stderr, "Spectra file %s: Couldn't open.\n", filename);
  return 1;
 }

 stat(filename, &_stat);
 size = _stat.st_size;

 fread(buffer, sizeof(char), size > PIPE_HEADERSIZE ? PIPE_HEADERSIZE : size, fp);

/* CHECK FILE SIZE */
 vp = (float *) ((void *) (buffer));

 byteswap = 0;

 if (*vp != PIPE_HEADER[0] || *(vp + 2) != PIPE_HEADER[2]) {
  byteswap = 1;
  swapbyte(sizeof(float), size > PIPE_HEADERSIZE ? PIPE_HEADERSIZE : size, buffer);

  vp = (float *) ((void *) (buffer));

  if (*vp != PIPE_HEADER[0] || *(vp + 2) != PIPE_HEADER[2]) {
   fprintf(stderr, "Spectra file: Not NMRPipe file.\n");
   goto escape;
  }
 }

 if (size != sizeof(float) * datasize[0] * datasize[1] + PIPE_HEADERSIZE) {
  fprintf(stderr, "Spectra file %s: Partially broken. (Actual=%d Expected=%d)\n", filename, (int) (size),
      (int) (sizeof(float)) * datasize[0] * datasize[1] + PIPE_HEADERSIZE);
  goto escape;
 }

 for (j = 0; j < datasize[1]; j++) {
  for (i = 0; i < datasize[0]; i++) {
   fseek(fp, (long) (PIPE_HEADERSIZE + (i + j * datasize[0]) * sizeof(float)), SEEK_SET);

   fread(&(mat2d[j][i]), sizeof(char), sizeof(float), fp);
  }
 }

 if (byteswap != 0)
  swapbyte(sizeof(float), datasize[0] * datasize[1] * sizeof(float), (char *) (&(mat2d[0][0])));

 fclose(fp);

 return 0;

escape:fclose(fp);

 return 1;
}

int openxyza4d(char spectra4d[], const int z, const int a, float **mat2d)
{
 struct stat _stat;
 FILE *fp;
 char filename[MAXLONGNAME], buffer[PIPE_HEADERSIZE];
 int i, j;
 long size = 0;
 float *vp;

 sprintf(filename, spectra4d, a + 1, z + 1);

 if ((fp = fopen(filename, "r")) == NULL) {
  fprintf(stderr, "Spectra file %s: Couldn't open.\n", filename);
  return 1;
 }

 stat(filename, &_stat);
 size = _stat.st_size;

 fread(buffer, sizeof(char), size > PIPE_HEADERSIZE ? PIPE_HEADERSIZE : size, fp);

/* CHECK FILE SIZE */
 vp = (float *) ((void *) (buffer));

 byteswap = 0;

 if (*vp != PIPE_HEADER[0] || *(vp + 2) != PIPE_HEADER[2]) {
  byteswap = 1;
  swapbyte(sizeof(float), size > PIPE_HEADERSIZE ? PIPE_HEADERSIZE : size, buffer);

  vp = (float *) ((void *) (buffer));

  if (*vp != PIPE_HEADER[0] || *(vp + 2) != PIPE_HEADER[2]) {
   fprintf(stderr, "Spectra file: Not NMRPipe file.\n");
   goto escape;
  }
 }

 if (size != sizeof(float) * datasize[0] * datasize[1] + PIPE_HEADERSIZE) {
  fprintf(stderr, "Spectra file %s: Partially broken. (Actual=%d Expected=%d)\n", filename, (int) (size),
      (int) (sizeof(float)) * datasize[0] * datasize[1] + PIPE_HEADERSIZE);
  goto escape;
 }

 for (j = 0; j < datasize[1]; j++) {
  for (i = 0; i < datasize[0]; i++) {
   fseek(fp, (long) (PIPE_HEADERSIZE + (i + j * datasize[0]) * sizeof(float)), SEEK_SET);

   fread(&(mat2d[j][i]), sizeof(char), sizeof(float), fp);
  }
 }

 if (byteswap != 0)
  swapbyte(sizeof(float), datasize[0] * datasize[1] * sizeof(float), (char *) (&(mat2d[0][0])));

 fclose(fp);

 return 0;

escape:fclose(fp);

 return 1;
}
