#include "xyza2pipe.h"

int pushadducsf2d(char spectra2d1[], char spectra2d2[], const float c1, const float c2, enum enum_combine_opr opr_code)
{
 int i, j;
 float **matrix2d_;

/* HEADER */
 fwrite(header, sizeof(char), PIPE_HEADERSIZE, stdout);

 fflush(stdout);

 matrix2d = fmalloc2d(datasize[1], datasize[0]);
 matrix2d_ = fmalloc2d(datasize[1], datasize[0]);

 if (openucsf2d(spectra2d1, matrix2d) != 0)
  goto escape;

 for (j = 0; j < datasize[1]; j++) {
  for (i = 0; i < datasize[0]; i++) {
   matrix2d_[j][i] = c1 * matrix2d[j][i];
  }
 }

 if (openucsf2d(spectra2d2, matrix2d) != 0)
  goto escape;

 switch (opr_code) {
 case COMBINE_ADD:
  for (j = 0; j < datasize[1]; j++) {
   for (i = 0; i < datasize[0]; i++) {
    matrix2d_[j][i] += c2 * matrix2d[j][i];
   }
  }

  break;

 case COMBINE_SUB:
  for (j = 0; j < datasize[1]; j++) {
   for (i = 0; i < datasize[0]; i++) {
    matrix2d_[j][i] -= c2 * matrix2d[j][i];
   }
  }

  break;

 case COMBINE_MUL:
  for (j = 0; j < datasize[1]; j++) {
   for (i = 0; i < datasize[0]; i++) {
    matrix2d_[j][i] *= c2 * matrix2d[j][i];
   }
  }

  break;

 case COMBINE_DIV:
  for (j = 0; j < datasize[1]; j++) {
   for (i = 0; i < datasize[0]; i++) {
    matrix2d_[j][i] /= c2 * matrix2d[j][i];
   }
  }

  break;
 }

 fpwrite2bin(stdout, &(matrix2d_[0][0]), datasize[1] * datasize[0]);

 fflush(stdout);

 free_fmatrix2d(matrix2d);
 free_fmatrix2d(matrix2d_);

 return 0;

escape:free_fmatrix2d(matrix2d);
 free_fmatrix2d(matrix2d_);

 return 1;
}

int pushadducsf3d(char spectra3d1[], char spectra3d2[], const float c1, const float c2, enum enum_combine_opr opr_code)
{
 int i, j, k;
 float **matrix2d_, ***matrix3d_;

/* HEADER */
 fwrite(header, sizeof(char), PIPE_HEADERSIZE, stdout);

 fflush(stdout);

 matrix2d_ = fmalloc2d(datasize[1], datasize[0]);
 matrix3d = fmalloc3d(datasize[2], datasize[1], datasize[0]);
 matrix3d_ = fmalloc3d(datasize[2], datasize[1], datasize[0]);

 if (openucsf3d(spectra3d1, matrix3d) != 0)
  goto escape;

 if (openucsf3d(spectra3d2, matrix3d_) != 0)
  goto escape;

 for (k = 0; k < datasize[2]; k++) {

  for (j = 0; j < datasize[1]; j++) {
   for (i = 0; i < datasize[0]; i++) {
    matrix2d_[j][i] = c1 * matrix3d[k][j][i];
   }
  }

  switch (opr_code) {
  case COMBINE_ADD:
   for (j = 0; j < datasize[1]; j++) {
    for (i = 0; i < datasize[0]; i++) {
     matrix2d_[j][i] += c2 * matrix3d_[k][j][i];
    }
   }

   break;

  case COMBINE_SUB:
   for (j = 0; j < datasize[1]; j++) {
    for (i = 0; i < datasize[0]; i++) {
     matrix2d_[j][i] -= c2 * matrix3d_[k][j][i];
    }
   }

   break;

  case COMBINE_MUL:
   for (j = 0; j < datasize[1]; j++) {
    for (i = 0; i < datasize[0]; i++) {
     matrix2d_[j][i] *= c2 * matrix3d_[k][j][i];
    }
   }

   break;

  case COMBINE_DIV:
   for (j = 0; j < datasize[1]; j++) {
    for (i = 0; i < datasize[0]; i++) {
     matrix2d_[j][i] /= c2 * matrix3d_[k][j][i];
    }
   }

   break;
  }

  fpwrite2bin(stdout, &(matrix2d_[0][0]), datasize[1] * datasize[0]);

  fflush(stdout);
 }

 free_fmatrix2d(matrix2d_);
 free_fmatrix3d(matrix3d);
 free_fmatrix3d(matrix3d_);

 return 0;

escape:free_fmatrix2d(matrix2d_);
 free_fmatrix3d(matrix3d);
 free_fmatrix3d(matrix3d_);

 return 1;
}

int pushadducsf4d(char spectra4d1[], char spectra4d2[], const float c1, const float c2, enum enum_combine_opr opr_code)
{
 int i, j, k, l;
 float **matrix2d_, ****matrix4d_;

/* HEADER */
 fwrite(header, sizeof(char), PIPE_HEADERSIZE, stdout);

 fflush(stdout);

 matrix2d_ = fmalloc2d(datasize[1], datasize[0]);
 matrix4d = fmalloc4d(datasize[3], datasize[2], datasize[1], datasize[0]);
 matrix4d_ = fmalloc4d(datasize[3], datasize[2], datasize[1], datasize[0]);

 if (openucsf4d(spectra4d1, matrix4d) != 0)
  goto escape;

 if (openucsf4d(spectra4d2, matrix4d_) != 0)
  goto escape;

 for (l = 0; l < datasize[3]; l++) {

  for (k = 0; k < datasize[2]; k++) {

   for (j = 0; j < datasize[1]; j++) {
    for (i = 0; i < datasize[0]; i++) {
     matrix2d_[j][i] = c1 * matrix4d[l][k][j][i];
    }
   }

   switch (opr_code) {
   case COMBINE_ADD:
    for (j = 0; j < datasize[1]; j++) {
     for (i = 0; i < datasize[0]; i++) {
      matrix2d_[j][i] += c2 * matrix4d_[l][k][j][i];
     }
    }

    break;

   case COMBINE_SUB:
    for (j = 0; j < datasize[1]; j++) {
     for (i = 0; i < datasize[0]; i++) {
      matrix2d_[j][i] -= c2 * matrix4d_[l][k][j][i];
     }
    }

    break;

   case COMBINE_MUL:
    for (j = 0; j < datasize[1]; j++) {
     for (i = 0; i < datasize[0]; i++) {
      matrix2d_[j][i] *= c2 * matrix4d_[l][k][j][i];
     }
    }

    break;

   case COMBINE_DIV:
    for (j = 0; j < datasize[1]; j++) {
     for (i = 0; i < datasize[0]; i++) {
      matrix2d_[j][i] /= c2 * matrix4d_[l][k][j][i];
     }
    }

    break;
   }

   fpwrite2bin(stdout, &(matrix2d_[0][0]), datasize[1] * datasize[0]);

   fflush(stdout);
  }
 }

 free_fmatrix2d(matrix2d_);
 free_fmatrix4d(matrix4d);
 free_fmatrix4d(matrix4d_);

 return 0;

escape:free_fmatrix2d(matrix2d_);
 free_fmatrix4d(matrix4d);
 free_fmatrix4d(matrix4d_);

 return 1;
}
