#include "xyza2pipe.h"

int pushaddvnmr2d(char spectra2d1[], char spectra2d2[], char pardir1[], char pardir2[], const float c1, const float c2,
    enum enum_combine_opr opr_code)
{
 int i, j;
 float **matrix2d_;

/* HEADER */
 fwrite(header, sizeof(char), PIPE_HEADERSIZE, stdout);

 fflush(stdout);

 matrix2d = fmalloc2d(datasize[1], datasize[0]);
 matrix2d_ = fmalloc2d(datasize[1], datasize[0]);

 if (openvnmr2d(spectra2d1, pardir1, matrix2d) != 0)
  goto escape;

 for (j = 0; j < datasize[1]; j++) {
  for (i = 0; i < datasize[0]; i++) {
   matrix2d_[j][i] = c1 * matrix2d[j][i];
  }
 }

 if (openvnmr2d(spectra2d2, pardir2, matrix2d) != 0)
  goto escape;

 switch (opr_code) {
 case COMBINE_ADD:
  for (j = 0; j < datasize[1]; j++) {
   for (i = 0; i < datasize[0]; i++) {
    matrix2d_[j][i] += c2 * matrix2d[j][i];
   }
  }

  break;

 case COMBINE_SUB:
  for (j = 0; j < datasize[1]; j++) {
   for (i = 0; i < datasize[0]; i++) {
    matrix2d_[j][i] -= c2 * matrix2d[j][i];
   }
  }

  break;

 case COMBINE_MUL:
  for (j = 0; j < datasize[1]; j++) {
   for (i = 0; i < datasize[0]; i++) {
    matrix2d_[j][i] *= c2 * matrix2d[j][i];
   }
  }

  break;

 case COMBINE_DIV:
  for (j = 0; j < datasize[1]; j++) {
   for (i = 0; i < datasize[0]; i++) {
    matrix2d_[j][i] /= c2 * matrix2d[j][i];
   }
  }

  break;
 }

 fpwrite2bin(stdout, &(matrix2d_[0][0]), datasize[1] * datasize[0]);

 fflush(stdout);

 free_fmatrix2d(matrix2d);
 free_fmatrix2d(matrix2d_);

 return 0;

escape:free_fmatrix2d(matrix2d);
 free_fmatrix2d(matrix2d_);

 return 1;
}

int pushaddvnmr3d(char spectra3d1[], char spectra3d2[], char pardir1[], char pardir2[], const int nfiles1,
    const int nfiles2, char suffix1[], char suffix2[], const float c1, const float c2, enum enum_combine_opr opr_code)
{
 char string[MAXCHAR];
 int i, j, k;
 float **matrix2d_, ***matrix3d_;

/* HEADER */
 fwrite(header, sizeof(char), PIPE_HEADERSIZE, stdout);

 fflush(stdout);

 matrix2d_ = fmalloc2d(datasize[1], datasize[0]);
 matrix3d = fmalloc3d(datasize[2], datasize[1], datasize[0]);
 matrix3d_ = fmalloc3d(datasize[2], datasize[1], datasize[0]);

 if (openvnmr3d(spectra3d1, pardir1, nfiles1, matrix3d) != 0)
  goto escape;

 sprintf(string, "rm -f monofile");
 strcat(string, suffix1);
 system(string);

 if (openvnmr3d(spectra3d2, pardir2, nfiles2, matrix3d_) != 0)
  goto escape;

 sprintf(string, "rm -f monofile");
 strcat(string, suffix2);
 system(string);

 for (k = 0; k < datasize[2]; k++) {

  for (j = 0; j < datasize[1]; j++) {
   for (i = 0; i < datasize[0]; i++) {
    matrix2d_[j][i] = c1 * matrix3d[k][j][i];
   }
  }

  switch (opr_code) {
  case COMBINE_ADD:
   for (j = 0; j < datasize[1]; j++) {
    for (i = 0; i < datasize[0]; i++) {
     matrix2d_[j][i] += c2 * matrix3d_[k][j][i];
    }
   }

   break;

  case COMBINE_SUB:
   for (j = 0; j < datasize[1]; j++) {
    for (i = 0; i < datasize[0]; i++) {
     matrix2d_[j][i] -= c2 * matrix3d_[k][j][i];
    }
   }

   break;

  case COMBINE_MUL:
   for (j = 0; j < datasize[1]; j++) {
    for (i = 0; i < datasize[0]; i++) {
     matrix2d_[j][i] *= c2 * matrix3d_[k][j][i];
    }
   }

   break;

  case COMBINE_DIV:
   for (j = 0; j < datasize[1]; j++) {
    for (i = 0; i < datasize[0]; i++) {
     matrix2d_[j][i] /= c2 * matrix3d_[k][j][i];
    }
   }

   break;
  }

  fpwrite2bin(stdout, &(matrix2d_[0][0]), datasize[1] * datasize[0]);

  fflush(stdout);
 }

 free_fmatrix2d(matrix2d_);
 free_fmatrix3d(matrix3d);
 free_fmatrix3d(matrix3d_);

 return 0;

escape:free_fmatrix2d(matrix2d_);
 free_fmatrix3d(matrix3d);
 free_fmatrix3d(matrix3d_);

 return 1;
}
