#include "xyza2pipe.h"

char header[PIPE_HEADERSIZE], _header_[PIPE_HEADERSIZE];
char axislabel[4][MAXASSNAME + 1];
char axisname[4][MAXASSNAME + 1];

short dimension, byteswap, headersize;
short swapdata = 0, swappar = 0, usrlabel = 0, usrshift = 0;
short leftcar = 0, extleft = 0, adjcar = 0, adjh2o = 0;

short blocksize[4];
short usrphase[4] = { 0, 0, 0, 0 };

int datasize[4];
int unitsize[4];

float obsfreq[4];
float spcenter[4];
float origfreq[4];
float spwidth[4];
float usrcenter[4];
float phase[4][2];

float **matrix2d;
float ***matrix3d;
float ****matrix4d;

const float PIPE_HEADER[3] = { PIPE_HEADER_0, PIPE_HEADER_1, PIPE_HEADER_2 };

const char *usage[] = {
 "xyza2pipe -i/--in inTemplate(2D/3D/4D) > stdout\n", "Optional Arguments:\n", " -x  Output X-Vector first\n",
     " -y  Output Y-Vector first\n", " -z  Output Z-Vector first\n", " -a  Output A-Vector first\n",
     " --xLAB X-Axis Label\n", " --yLAB Y-Axis Label\n", " --zLAB Z-Axis Label\n", " --aLAB A-Axis Label\n",
     " --xCAR X-Axis Center [ppm]\n", " --yCAR Y-Axis Center [ppm]\n", " --zCAR Z-Axis Center [ppm]\n",
     " --aCAR A-Axis Center [ppm]\n\n", ""
};

int main(int argc, char *argv[])
{
 char axis_option, filename[MAXLONGNAME];
 int l, opt_idx = 0;

 static struct option options[] = {
  {"in", required_argument, 0, 'i'},
  {"x", no_argument, 0, 'x'},
  {"y", no_argument, 0, 'y'},
  {"z", no_argument, 0, 'z'},
  {"a", no_argument, 0, 'a'},
  {"xLAB", required_argument, 0, 6},
  {"yLAB", required_argument, 0, 7},
  {"zLAB", required_argument, 0, 8},
  {"aLAB", required_argument, 0, 9},
  {"xCAR", required_argument, 0, 10},
  {"yCAR", required_argument, 0, 11},
  {"zCAR", required_argument, 0, 12},
  {"aCAR", required_argument, 0, 13},
  {0, 0, 0, 0}
 };

 filename[0] = 0;
 axis_option = 'x';

 for (l = 0; l < 4; l++)
  usrcenter[l] = NULLPPM;

 while (1) {
  l = getopt_long(argc, argv, "i:xyza", options, &opt_idx);

  if (l == -1)
   break;

  switch (l) {
  case 0:
   if (options[opt_idx].flag != 0)
    break;
   fprintf(stderr, "option %s", options[opt_idx].name);
   if (optarg)
    fprintf(stderr, " with arg %s", optarg);
   fputc('\n', stderr);
   break;
  case 'i':
   strncpy(filename, optarg, MAXLONGNAME);
   break;
  case 'x':
   axis_option = 'x';
   break;
  case 'y':
   axis_option = 'y';
   break;
  case 'z':
   axis_option = 'z';
   break;
  case 'a':
   axis_option = 'a';
   break;
  case 6:
   usrlabel = 1;
   strncpy(axislabel[0], optarg, MAXASSNAME);
   break;
  case 7:
   usrlabel = 1;
   strncpy(axislabel[1], optarg, MAXASSNAME);
   break;
  case 8:
   usrlabel = 1;
   strncpy(axislabel[2], optarg, MAXASSNAME);
   break;
  case 9:
   usrlabel = 1;
   strncpy(axislabel[3], optarg, MAXASSNAME);
   break;
  case 10:
   usrshift = 1;
   usrcenter[0] = atof(optarg);
   break;
  case 11:
   usrshift = 1;
   usrcenter[1] = atof(optarg);
   break;
  case 12:
   usrshift = 1;
   usrcenter[2] = atof(optarg);
   break;
  case 13:
   usrshift = 1;
   usrcenter[3] = atof(optarg);
   break;
  case '?':			/* getopt_long already printed an error message. */
   break;
  }
 }

 if (argc == 2)
  strncpy(filename, argv[1], MAXLONGNAME);

 else if (optind < argc) {
  fprintf(stderr, "non-option ARGV-elements: ");
  while (optind < argc)
   fprintf(stderr, "%s ", argv[optind++]);
  fputc('\n', stderr);
 }

 if (filename[0] == 0) {
  l = 0;
  while (strcmp(usage[l], "") != 0)
   fprintf(stderr, "%s", usage[l++]);

  return 1;
 }

 if (checkxyza(filename) != 0)
  return 1;

 cnvhdr(axis_option, 'f');

 if (isatty(STDOUT_FILENO)) {
  fprintf(stderr, "xyza2pipe error: output to terminal.\n");
  return 1;
 }

 switch (dimension) {
 case 2:
  return pushxyza2d(filename, axis_option);
  break;

 case 3:
  return pushxyza3d(filename, axis_option);
  break;

 case 4:
  return pushxyza4d(filename, axis_option);
  break;
 }

 return 0;
}
